import { documentHighlightKindNames } from '../../../lsp';
import { CodeMirrorLSPFeature } from '../feature';
export class Highlights extends CodeMirrorLSPFeature {
    constructor() {
        super(...arguments);
        this.name = 'Highlights';
        this.highlight_markers = [];
    }
    register() {
        this.connection_handlers.set('highlight', this.handleHighlight.bind(this));
        this.editor_handlers.set('cursorActivity', this.onCursorActivity.bind(this));
        super.register();
    }
    remove() {
        super.remove();
        this.clear_markers();
    }
    clear_markers() {
        for (let marker of this.highlight_markers) {
            marker.clear();
        }
        this.highlight_markers = [];
    }
    handleHighlight(items) {
        this.clear_markers();
        if (!items) {
            return;
        }
        for (let item of items) {
            let range = this.range_to_editor_range(item.range);
            let kind_class = item.kind
                ? 'cm-lsp-highlight-' + documentHighlightKindNames[item.kind]
                : '';
            let marker = this.highlight_range(range, 'cm-lsp-highlight ' + kind_class);
            this.highlight_markers.push(marker);
        }
    }
    onCursorActivity() {
        let root_position = this.virtual_editor
            .getDoc()
            .getCursor('start');
        let document;
        try {
            document = this.virtual_editor.document_at_root_position(root_position);
        }
        catch (e) {
            console.warn('Could not obtain virtual document from position', root_position);
            return;
        }
        if (document !== this.virtual_document) {
            return;
        }
        try {
            let virtual_position = this.virtual_editor.root_position_to_virtual_position(root_position);
            this.connection.getDocumentHighlights(virtual_position);
        }
        catch (e) {
            console.warn('Could not get highlights:', e);
        }
    }
}
Highlights.commands = [
    {
        id: 'highlight-references',
        execute: ({ connection, virtual_position }) => connection.getReferences(virtual_position),
        is_enabled: ({ connection }) => connection.isReferencesSupported(),
        label: 'Highlight references'
    },
    {
        id: 'highlight-type-definition',
        execute: ({ connection, virtual_position }) => connection.getTypeDefinition(virtual_position),
        is_enabled: ({ connection }) => connection.isTypeDefinitionSupported(),
        label: 'Highlight type definition'
    }
];
//# sourceMappingURL=highlights.js.map