import { VirtualDocument } from './virtual/document';
import { LSPConnection } from './connection';
import { Signal } from '@phosphor/signaling';
export interface IDocumentConnectionData {
    virtual_document: VirtualDocument;
    connection: LSPConnection;
}
interface ISocketConnectionOptions {
    virtual_document: VirtualDocument;
    /**
     * The language identifier, corresponding to the API endpoint on the LSP proxy server.
     */
    language: string;
    /**
     * The root path in the JupyterLab (virtual) path space
     */
    root_path: string;
    /**
     * The real root path (on the server), if exposed by the server. If present, it has to be an absolute path.
     */
    server_root: string | null;
    /**
     * Path to the document in the JupyterLab space
     */
    document_path: string;
}
/**
 * Each Widget with a document (whether file or a notebook) has its own DocumentConnectionManager
 * (see JupyterLabWidgetAdapter), keeping the virtual document spaces separate if a file is opened twice.
 */
export declare class DocumentConnectionManager {
    connections: Map<VirtualDocument.id_path, LSPConnection>;
    documents: Map<VirtualDocument.id_path, VirtualDocument>;
    initialized: Signal<DocumentConnectionManager, IDocumentConnectionData>;
    connected: Signal<DocumentConnectionManager, IDocumentConnectionData>;
    /**
     * Connection temporarily lost or could not be fully established; a re-connection will be attempted;
     */
    disconnected: Signal<DocumentConnectionManager, IDocumentConnectionData>;
    /**
     * Connection was closed permanently and no-reconnection will be attempted, e.g.:
     *  - there was a serious server error
     *  - user closed the connection,
     *  - re-connection attempts exceeded,
     */
    closed: Signal<DocumentConnectionManager, IDocumentConnectionData>;
    documents_changed: Signal<DocumentConnectionManager, Map<VirtualDocument.id_path, VirtualDocument>>;
    private ignored_languages;
    constructor();
    connect_document_signals(virtual_document: VirtualDocument): void;
    private connect_socket;
    retry_to_connect(options: ISocketConnectionOptions, reconnect_delay: number, retrials_left?: number): Promise<void>;
    connect(options: ISocketConnectionOptions): Promise<LSPConnection>;
    close_all(): void;
}
export declare namespace DocumentConnectionManager {
    function solve_uris(virtual_document: VirtualDocument, language: string): {
        base: string;
        document: string;
        server: string;
        socket: string;
    };
}
export {};
